const Discord = require('discord.js')
const { MessageEmbed } = require('discord.js')
const { MessageButton, MessageActionRow } = require('discord-buttons')
const ms = require('ms')
const GameManager = new Map()

module.exports = async(options) => {
	if (!options.message) {
		throw new Error('NPM Error: message debe ser definido');
	}
	if (typeof options.message !== 'object') {
		throw new TypeError('NPM Error: se ha introducido un mensaje de discord invalido');
	}

	if (!options.embed) options.embed = {};

	if (typeof options.embed !== 'object') {
		throw new TypeError('NPM Error: el embed debe ser un objeto!');
	}

	if (!options.embed.color) {
		options.embed.color = 'RANDOM'
	}
	if (typeof options.embed.color !== 'string') {
		throw new TypeError('NPM Error: el color del embed debe ser un string!');
	}

	if (!options.embed.timestamp) {
		options.embed.timestamp = false;
	}
	if (typeof options.embed.timestamp !== 'boolean') {
		throw new TypeError('NPM Error: el timestamp del embed debe ser un boolean!');
	}

	if (!options.winMessage) {
		options.winMessage = 'GG, {author} has contestado correctamente a la trivia!';
	}

	if (typeof options.winMessage !== 'string') {
		throw new TypeError('NPM Error: winMessage debe ser un string!');
	}

	if (!options.othersMessage) {
		options.othersMessage = 'Solo {author} puede usar los botones!';
	}
	if (typeof options.othersMessage !== 'string') {
		throw new TypeError('NPM Error: otherUser debe ser un string.');
	}

	const author = options.message.author;

	const inGameMessage = options.inGame.replace(/{author}/, `${options.message.author}`)

	if(GameManager.has(`active-${author.id}`)) return options.message.lineReply(inGameMessage)

	const preguntas = require('../data/preguntas.json')

	const randoms = require('../data/randoms.json')


	const button1 = new MessageButton()
	.setLabel('1')
	.setStyle('blurple')
	.setID(`option1-${options.message.id}`)

	const button2 = new MessageButton()
	.setLabel('2')
	.setStyle('blurple')
	.setID(`option2-${options.message.id}`)

	const button3 = new MessageButton()
	.setLabel('3')
	.setStyle('blurple')
	.setID(`option3-${options.message.id}`)

	const buttons = new MessageActionRow()
	.addComponent(button1)
	.addComponent(button2)
	.addComponent(button3);


	const qr = preguntas[Math.floor(Math.random() * preguntas.length)]

	const question = qr.question
	const answer = qr.answer

	let ranswer1 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	let ranswer2 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === answer) ranswer1 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === answer) ranswer1 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === ranswer2) ranswer2 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === ranswer2) ranswer2 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === answer) ranswer2 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	if(ranswer1 === answer) ranswer2 = randoms.randoms[Math.floor(Math.random() * randoms.randoms.length)]

	let questionsr = [`${answer}`, `${ranswer1}`, `${ranswer2}`]

	const questions = questionsr.sort(() => Math.random() - 0.5);

	
	let textattempts;

	if(options.attemps === 1) textattempts = `${options.attempts} intento`
	if(options.attemps === 2) textattempts = `${options.attempts} intentos`

	const embed = new MessageEmbed()
	.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
	.setDescription(`Responde bien a la siguiente pregunta\nPregunta: ${question}`)
	.addField('\u200b', [
			`1. ${questions[0]}`,
			`2. ${questions[1]}`,
			`3. ${questions[2]}`,
			`\u200b`
		]
	)
	.setColor(options.embed.color)
	.setFooter(`Recuerda que tienes ${options.time}`)

	if(options.embed.timestamp === true) embed.setTimestamp()


	let msg = await options.message.lineReply(embed)

	msg.edit(embed, buttons)

	await GameManager.set(`active-${options.message.author.id}`, true)

	const collector = msg.createButtonCollector((button) => button, { time: ms(options.time) })

	const replynouser = options.otherUser.replace(/{author}/gm, `${options.message.author}`)

	collector.on("collect", async(button) => {
		if(button.clicker.user.id !== options.message.author.id) return button.reply.send(replynouser, true)

		if(button.id === `option1-${options.message.id}`) {
			const answeruser = questions[0];

			if(answer === answeruser) {
				const button1yes = new MessageButton().setLabel('1').setStyle('green').setID(`option1-${options.message.id}`).setDisabled()
				const button2yes = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
				const button3yes = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()

				const buttonsyes = new MessageActionRow().addComponent(button1yes).addComponent(button2yes).addComponent(button3yes);

				const embedyes = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`GG! La respuesta correcta es **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`**1. ${questions[0]}**`,
							`2. ${questions[1]}`,
							`3. ${questions[2]}`,
							`\u200b`
						]
					)
					.setColor(options.embed.color)

				if(options.embed.timestamp === true) embedyes.setTimestamp()

				msg.edit(embedyes, buttonsyes)

				await GameManager.delete(`active-${options.message.author.id}`)

				await GameManager.delete(`attempts-${options.message.author.id}`)

				const response = options.winMessage.replace(/{author}/gm, `${options.message.author}`)

				button.reply.send(response)
			} else if(answer !== answeruser) {
				const index = questions.findIndex(x => x === answer)

				let button1no;
				let button2no;
				let button3no;

				if(index === 0) {
					button1no = new MessageButton().setLabel('1').setStyle('green').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 1) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('green').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 2) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('green').setID(`option3-${options.message.id}`).setDisabled()
				}

				const buttonsno = new MessageActionRow().addComponent(button1no).addComponent(button2no).addComponent(button3no);

				const embedno = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`¡Perdiste! La respuesta correcta era: **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`1. ${questions[0]}`,
							`2. ${questions[1]}`,
							`3. ${questions[2]}`,
							`\u200b`
						]
					)
					.setColor(options.embed.color)
					.setFooter(`Recuerda que tienes ${options.time}`)

				if(options.embed.timestamp === true) embedno.setTimestamp()

				msg.edit(embedno, buttonsno)

				const responsefail = options.loseMessage.replace(/{author}/gm, `${options.message.author}`).replace(/{answer}/gm, `${answer}`)

				button.reply.send(responsefail)

				await GameManager.delete(`active-${options.message.author.id}`)
			}
		}

		if(button.id === `option2-${options.message.id}`) {
			const answeruser = questions[1];

			if(answer === answeruser) {
				const button1yes = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
				const button2yes = new MessageButton().setLabel('2').setStyle('green').setID(`option2-${options.message.id}`).setDisabled()
				const button3yes = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()

				const buttonsyes = new MessageActionRow().addComponent(button1yes).addComponent(button2yes).addComponent(button3yes);

				const embedyes = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`GG! La respuesta correcta es **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`1. ${questions[0]}`,
							`**2. ${questions[1]}**`,
							`3. ${questions[2]}`,
							`\u200b`
						]
					)
					.setColor(options.embed.color)

				if(options.embed.timestamp === true) embedyes.setTimestamp()

				msg.edit(embedyes, buttonsyes)

				const response = options.winMessage.replace(/{author}/gm, `${options.message.author}`)

				await GameManager.delete(`active-${options.message.author.id}`)

				return button.reply.send(response)
			} else if(answer !== answeruser) {
				const index = questions.findIndex(x => x === answer)

				let button1no;
				let button2no;
				let button3no;

				if(index === 0) {
					button1no = new MessageButton().setLabel('1').setStyle('green').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 1) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('green').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 2) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('green').setID(`option3-${options.message.id}`).setDisabled()
				}

				const buttonsno = new MessageActionRow().addComponent(button1no).addComponent(button2no).addComponent(button3no);

				const embedno = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`¡Perdiste! La respuesta correcta era: **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`1. ${questions[0]}`,
							`2. ${questions[1]}`,
							`3. ${questions[2]}`,
							`\u200b`
							]
					)
					.setColor(options.embed.color)

				if(options.embed.timestamp === true) embedno.setTimestamp()

				msg.edit(embedno, buttonsno)

				const responsefail = options.loseMessage.replace(/{author}/gm, `${options.message.author}`).replace(/{answer}/gm, `${answer}`)

				button.reply.send(responsefail)

					GameManager.delete(`active-${options.message.author.id}`)
			}
		}

		if(button.id === `option3-${options.message.id}`) {
			const answeruser = questions[2];

			if(answer === answeruser) {
				const button1yes = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
				const button2yes = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()	
				const button3yes = new MessageButton().setLabel('3').setStyle('green').setID(`option3-${options.message.id}`).setDisabled()

				const buttonsyes = new MessageActionRow().addComponent(button1yes).addComponent(button2yes).addComponent(button3yes);

				const embedyes = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`GG! La respuesta correcta es **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`1. ${questions[0]}`,
							`2. ${questions[1]}`,
							`**3. ${questions[2]}**`,
							`\u200b`
						]
					)
					.setColor(options.embed.color)

				if(options.embed.timestamp === true) embedyes.setTimestamp()

				msg.edit(embedyes, buttonsyes)

				const response = options.winMessage.replace(/{author}/gm, `${options.message.author}`)

				await GameManager.delete(`active-${options.message.author.id}`)

				button.reply.send(response)
			} else if(answer !== answeruser) {
				const index = questions.findIndex(x => x === answer)

				let button1no;
				let button2no;
				let button3no;

				if(index === 0) {
					button1no = new MessageButton().setLabel('1').setStyle('green').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 1) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('green').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('red').setID(`option3-${options.message.id}`).setDisabled()
				} else if(index === 2) {
					button1no = new MessageButton().setLabel('1').setStyle('red').setID(`option1-${options.message.id}`).setDisabled()
					button2no = new MessageButton().setLabel('2').setStyle('red').setID(`option2-${options.message.id}`).setDisabled()
					button3no = new MessageButton().setLabel('3').setStyle('green').setID(`option3-${options.message.id}`).setDisabled()
				}

				const buttonsno = new MessageActionRow().addComponent(button1no).addComponent(button2no).addComponent(button3no);

				const embedno = new MessageEmbed()
					.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
					.setDescription(`¡Perdiste! La respuesta correcta era: **${answer}**\nLa Pregunta: ${question}`)
					.addField('\u200b', [
							`1. ${questions[0]}`,
							`2. ${questions[1]}`,
							`3. ${questions[2]}`,
							`\u200b`
						]
					)
					.setColor(options.embed.color)

				if(options.embed.timestamp === true) embedno.setTimestamp()

				msg.edit(embedno, buttonsno)

				const responsefail = options.loseMessage.replace(/{author}/gm, `${options.message.author}`).replace(/{answer}/gm, `${answer}`)

				button.reply.send(responsefail)

				GameManager.delete(`active-${options.message.author.id}`)
				
			}
		}
	});

	collector.on("end", async(button) => {
		if(GameManager.has(`active-${author.id}`)) {
			const button1end = new MessageButton().setLabel('1').setStyle('gray').setID(`option1-${options.message.id}`).setDisabled()
			const button2end = new MessageButton().setLabel('2').setStyle('gray').setID(`option2-${options.message.id}`).setDisabled()
			const button3end = new MessageButton().setLabel('3').setStyle('gray').setID(`option3-${options.message.id}`).setDisabled()

			const buttonsend = new MessageActionRow().addComponent(button1end).addComponent(button2end).addComponent(button3end);

			const embedend = new MessageEmbed()
				.setAuthor(author.tag, author.displayAvatarURL({ dynamic: true }))
				.setDescription(`¡Se ha excedido el tiempo limite de respuesta!\nLa Pregunta: ${question}`)
				.addField('\u200b', [
						`1. ${questions[0]}`,
						`2. ${questions[1]}`,
						`3. ${questions[2]}`,
						`\u200b`
					]
				)
				.setColor(options.embed.color)
				.setFooter(`Recuerda que tienes ${options.time}`)

			if(options.embed.timestamp === true) embedend.setTimestamp()

			msg.edit(embedend, buttonsend)

		} else if(!GameManager.has(`active-${author.id}`)) return;
	})
}

